@@ -3,8 +3,8 @@ from unittest.mock import patch
 
 import pytest
 
+from .fixtures.http_server import *
 from .fixtures.simple_app import *
-from .fixtures.sse_server import *
 
 
 class MockContext:
@@ -27,23 +27,33 @@ def mock_context():
 
 @pytest.fixture
 def simulation_env():
+    from veris_ai import veris
+
+    # Set session_id to enable simulation mode
+    veris.set_session_id("test-session-123")
+
     with patch.dict(
         os.environ,
         {
-            "VERIS_MOCK_ENDPOINT_URL": "http://test-endpoint",
-            "ENV": "simulation",
+            "VERIS_ENDPOINT_URL": "http://test-endpoint",
         },
     ):
         yield
+        # Clean up session_id after test
+        veris.clear_session_id()
 
 
 @pytest.fixture
 def production_env():
+    from veris_ai import veris
+
+    # Clear session_id to ensure production mode (no simulation)
+    veris.clear_session_id()
+
     with patch.dict(
         os.environ,
         {
-            "VERIS_MOCK_ENDPOINT_URL": "http://test-endpoint",
-            "ENV": "production",
+            "VERIS_ENDPOINT_URL": "http://test-endpoint",
         },
     ):
         yield
