@@ -7,6 +7,7 @@ import httpx
 import pytest
 
 from veris_ai import veris
+from veris_ai.api_client import get_api_client
 from veris_ai.tool_mock import _session_id_context
 
 
@@ -24,15 +25,7 @@ async def test_mock_decorator_simulation_mode(simulation_env):
 
     mock_response = {"result": {"mocked": True}}
 
-    with patch("httpx.AsyncClient") as mock_client:
-        mock_response_obj = Mock()
-        mock_response_obj.json.return_value = mock_response
-        mock_response_obj.raise_for_status.return_value = None
-
-        mock_client.return_value.__aenter__.return_value.post = AsyncMock(
-            return_value=mock_response_obj,
-        )
-
+    with patch.object(get_api_client(), "post", return_value=mock_response):
         result = await test_func("test", 42)
         assert result == {"result": {"mocked": True}}
 
@@ -55,42 +48,31 @@ async def test_mock_with_context(simulation_env):
 
     mock_response = {"result": {"mocked": True}}
 
-    with patch("veris_ai.tool_mock.httpx.AsyncClient") as mock_client:
-        mock_response_obj = Mock()
-        mock_response_obj.json.return_value = mock_response
-        mock_response_obj.raise_for_status.return_value = None
-
-        mock_client.return_value.__aenter__.return_value.post = AsyncMock(
-            return_value=mock_response_obj,
-        )
+    with patch.object(get_api_client(), "post") as mock_request:
+        mock_request.return_value = mock_response
         veris.set_session_id("test-session-id")
         result = await test_func()
-        first_call = mock_client.return_value.__aenter__.return_value.post.call_args
-        assert first_call.kwargs["json"]["session_id"] == "test-session-id"
+        # Check that the mock was called with correct session_id
+        mock_request.assert_called_once()
+        call_args = mock_request.call_args
+        assert call_args[0][1]["session_id"] == "test-session-id"  # payload is second arg
         assert result == {"result": {"mocked": True}}
 
 
 @pytest.mark.asyncio
-async def test_mock_without_context(simulation_env):
+async def test_mock_without_context():
+    """Test that without session_id, the function runs in production mode."""
+
     @veris.mock(mode="function")
     async def test_func() -> dict:
         return {"result": "real"}
 
-    mock_response = {"result": {"mocked": True}}
+    # Clear session_id to ensure production mode
+    veris.clear_session_id()
 
-    with patch("veris_ai.tool_mock.httpx.AsyncClient") as mock_client:
-        mock_response_obj = Mock()
-        mock_response_obj.json.return_value = mock_response
-        mock_response_obj.raise_for_status.return_value = None
-
-        mock_client.return_value.__aenter__.return_value.post = AsyncMock(
-            return_value=mock_response_obj,
-        )
-        veris.clear_session_id()
-        result = await test_func()
-        first_call = mock_client.return_value.__aenter__.return_value.post.call_args
-        assert first_call.kwargs["json"]["session_id"] == None
-        assert result == {"result": {"mocked": True}}
+    # In production mode, the original function should be called
+    result = await test_func()
+    assert result == {"result": "real"}
 
 
 # Test error handling
@@ -100,30 +82,34 @@ async def test_mock_http_error(simulation_env):
     async def test_func(param1: str) -> dict:
         return {"result": "real"}
 
-    with patch("httpx.AsyncClient") as mock_client:
-        mock_client.return_value.__aenter__.return_value.post = AsyncMock(
-            side_effect=httpx.HTTPError("Mock HTTP Error"),
-        )
-
-        with pytest.raises(httpx.HTTPError):
-            await test_func("test")
+    with (
+        patch.object(get_api_client(), "post", side_effect=httpx.HTTPError("Mock HTTP Error")),
+        pytest.raises(httpx.HTTPError),
+    ):
+        await test_func("test")
 
 
 @pytest.mark.asyncio
 async def test_mock_missing_endpoint():
-    with (
-        patch.dict(os.environ, {"VERIS_MOCK_ENDPOINT_URL": ""}),
-        pytest.raises(ValueError, match="VERIS_MOCK_ENDPOINT_URL environment variable is not set"),
-    ):
+    """Test that missing endpoint raises ValueError when function is called in simulation mode."""
+    with patch.dict(os.environ, {"VERIS_API_URL": ""}):
 
         @veris.mock()
         async def test_func():
             return {"result": "real"}
 
+        # Set session_id to enable simulation mode
+        veris.set_session_id("test-session")
+
+        # Since VERIS_API_URL has a default value, the function will try to connect
+        # We'll test that the connection fails with httpx.ConnectError
+        with pytest.raises(httpx.ConnectError):
+            await test_func()
+
 
 @pytest.mark.asyncio
 async def test_mock_invalid_endpoint(simulation_env):
-    with patch.dict(os.environ, {"VERIS_MOCK_ENDPOINT_URL": ""}), pytest.raises(ValueError):
+    with patch.dict(os.environ, {"VERIS_API_URL": ""}), pytest.raises(httpx.ConnectError):
 
         @veris.mock()
         async def test_func():
@@ -140,15 +126,7 @@ async def test_mock_string_json_response(simulation_env):
 
     mock_response = {"result": '{"key": "value"}'}
 
-    with patch("httpx.AsyncClient") as mock_client:
-        mock_response_obj = Mock()
-        mock_response_obj.json.return_value = mock_response
-        mock_response_obj.raise_for_status.return_value = None
-
-        mock_client.return_value.__aenter__.return_value.post = AsyncMock(
-            return_value=mock_response_obj,
-        )
-
+    with patch.object(get_api_client(), "post", return_value=mock_response):
         result = await test_func()
         assert result == {"result": '{"key": "value"}'}
 
@@ -192,32 +170,25 @@ async def test_context_var_persistence_in_call(simulation_env):
 
     mock_response = {"result": {"mocked": True}}
 
-    with patch("veris_ai.tool_mock.httpx.AsyncClient") as mock_client:
-        mock_response_obj = Mock()
-        mock_response_obj.json.return_value = mock_response
-        mock_response_obj.raise_for_status.return_value = None
-
-        async def mock_post(*args, **kwargs):
-            # Capture session_id during HTTP call
-            captured_session_ids.append(veris.session_id)
-            return mock_response_obj
-
-        mock_client.return_value.__aenter__.return_value.post = AsyncMock(side_effect=mock_post)
+    def mock_request_func(endpoint, payload):
+        # Capture session_id during HTTP call
+        captured_session_ids.append(veris.session_id)
+        return mock_response
 
+    with patch.object(get_api_client(), "post", side_effect=mock_request_func) as mock_request:
         # Set session_id and call function
         veris.set_session_id("test-session-123")
         await test_func()
 
         # Verify session_id was captured
-        # In simulation mode, we should have 2 captures (one in function, one in HTTP call)
-        # But if the function ran in production mode, we'd only have 1
+        # In simulation mode, we should have captures from the HTTP call
         assert len(captured_session_ids) >= 1
         assert captured_session_ids[0] == "test-session-123"
 
         # Verify the HTTP client was called with correct session_id
-        mock_client.return_value.__aenter__.return_value.post.assert_called_once()
-        call_args = mock_client.return_value.__aenter__.return_value.post.call_args
-        assert call_args[1]["json"]["session_id"] == "test-session-123"
+        mock_request.assert_called_once()
+        call_args = mock_request.call_args
+        assert call_args[0][1]["session_id"] == "test-session-123"
 
 
 @pytest.mark.asyncio
@@ -261,6 +232,176 @@ async def test_context_var_no_interference():
     assert ("task2-after", "task2-session") in results
 
 
+# Test spy decorator
+@pytest.mark.asyncio
+async def test_spy_decorator_with_session(simulation_env):
+    """Test that spy decorator logs function calls and returns real results."""
+
+    @veris.spy()
+    async def test_func(x: int, y: str) -> dict:
+        """A test function that returns a dict."""
+        return {"value": x, "message": y}
+
+    # Mock the logging functions
+    with (
+        patch("veris_ai.tool_mock.log_tool_call") as mock_log_call,
+        patch("veris_ai.tool_mock.log_tool_response") as mock_log_response,
+    ):
+        veris.set_session_id("spy-session-123")
+        result = await test_func(42, "hello")
+
+        # Verify the original function was executed
+        assert result == {"value": 42, "message": "hello"}
+
+        # Verify logging was called
+        mock_log_call.assert_called_once_with(
+            session_id="spy-session-123",
+            function_name="test_func",
+            parameters={
+                "x": {"value": "42", "type": "<class 'int'>"},
+                "y": {"value": "hello", "type": "<class 'str'>"},
+            },
+            docstring="A test function that returns a dict.",
+        )
+        mock_log_response.assert_called_once_with(
+            session_id="spy-session-123", response={"value": 42, "message": "hello"}
+        )
+
+
+@pytest.mark.asyncio
+async def test_spy_decorator_without_session():
+    """Test that spy decorator executes original function when no session_id."""
+
+    @veris.spy()
+    async def test_func(x: int) -> int:
+        return x * 2
+
+    # Clear session to ensure no spying
+    veris.clear_session_id()
+
+    with (
+        patch("veris_ai.tool_mock.log_tool_call") as mock_log_call,
+        patch("veris_ai.tool_mock.log_tool_response") as mock_log_response,
+    ):
+        result = await test_func(21)
+
+        # Verify the original function was executed
+        assert result == 42
+
+        # Verify logging was NOT called
+        mock_log_call.assert_not_called()
+        mock_log_response.assert_not_called()
+
+
+def test_spy_decorator_sync_function(simulation_env):
+    """Test spy decorator with synchronous functions."""
+
+    @veris.spy()
+    def sync_func(a: int, b: int) -> int:
+        """Adds two numbers."""
+        return a + b
+
+    with (
+        patch("veris_ai.tool_mock.log_tool_call") as mock_log_call,
+        patch("veris_ai.tool_mock.log_tool_response") as mock_log_response,
+    ):
+        veris.set_session_id("sync-spy-session")
+        result = sync_func(10, 20)
+
+        # Verify the original function was executed
+        assert result == 30
+
+        # Verify logging was called
+        mock_log_call.assert_called_once_with(
+            session_id="sync-spy-session",
+            function_name="sync_func",
+            parameters={
+                "a": {"value": "10", "type": "<class 'int'>"},
+                "b": {"value": "20", "type": "<class 'int'>"},
+            },
+            docstring="Adds two numbers.",
+        )
+        mock_log_response.assert_called_once_with(session_id="sync-spy-session", response=30)
+
+
+def test_spy_decorator_sync_without_session():
+    """Test sync spy decorator without session executes normally."""
+
+    @veris.spy()
+    def sync_func(x: str) -> str:
+        return x.upper()
+
+    veris.clear_session_id()
+
+    with (
+        patch("veris_ai.tool_mock.log_tool_call") as mock_log_call,
+        patch("veris_ai.tool_mock.log_tool_response") as mock_log_response,
+    ):
+        result = sync_func("hello")
+
+        # Verify the original function was executed
+        assert result == "HELLO"
+
+        # Verify logging was NOT called
+        mock_log_call.assert_not_called()
+        mock_log_response.assert_not_called()
+
+
+@pytest.mark.asyncio
+async def test_spy_decorator_preserves_metadata():
+    """Test that spy decorator preserves function metadata."""
+
+    @veris.spy()
+    async def documented_spy_function(x: int, y: str = "default") -> str:
+        """This is a documented spy function."""
+        return f"{x}: {y}"
+
+    # Check function metadata is preserved
+    assert documented_spy_function.__name__ == "documented_spy_function"
+    assert documented_spy_function.__doc__ == "This is a documented spy function."
+
+    # Test that it works correctly
+    veris.set_session_id("test-spy")
+    result = await documented_spy_function(100)
+    assert result == "100: default"
+
+
+@pytest.mark.asyncio
+async def test_spy_decorator_with_exception():
+    """Test that spy decorator logs and re-raises exceptions."""
+
+    @veris.spy()
+    async def failing_func(x: int) -> int:
+        """A function that raises an exception."""
+        if x < 0:
+            raise ValueError("Negative value not allowed")
+        return x * 2
+
+    with (
+        patch("veris_ai.tool_mock.log_tool_call") as mock_log_call,
+        patch("veris_ai.tool_mock.log_tool_response") as mock_log_response,
+    ):
+        veris.set_session_id("exception-spy-session")
+
+        # Test normal execution
+        result = await failing_func(5)
+        assert result == 10
+        mock_log_call.assert_called()
+        mock_log_response.assert_called()
+
+        # Reset mocks
+        mock_log_call.reset_mock()
+        mock_log_response.reset_mock()
+
+        # Test exception case
+        with pytest.raises(ValueError, match="Negative value not allowed"):
+            await failing_func(-1)
+
+        # Verify logging was still called for the call
+        mock_log_call.assert_called_once()
+        # Response logging might not be called on exception, that's okay
+
+
 # Test stub decorator
 @pytest.mark.asyncio
 async def test_stub_decorator_simulation_mode(simulation_env):
@@ -352,19 +493,11 @@ async def test_mock_decorator_tool_mode(simulation_env):
     async def test_func() -> dict:
         return {"result": "real"}
 
-    mock_response = "This is a mocked tool response"
-
-    with patch("veris_ai.tool_mock.httpx.AsyncClient") as mock_client:
-        mock_response_obj = Mock()
-        mock_response_obj.json.return_value = mock_response
-        mock_response_obj.raise_for_status.return_value = None
-
-        mock_client.return_value.__aenter__.return_value.post = AsyncMock(
-            return_value=mock_response_obj,
-        )
+    mock_response = {"result": "mocked"}
 
+    with patch.object(get_api_client(), "post", return_value=mock_response):
         result = await test_func()
-        assert result == {"content": [{"type": "text", "text": mock_response}]}
+        assert result == {"result": "mocked"}
 
 
 @pytest.mark.asyncio
@@ -377,15 +510,7 @@ async def test_mock_decorator_function_mode(simulation_env):
 
     mock_response = {"mocked": True, "data": [1, 2, 3]}
 
-    with patch("veris_ai.tool_mock.httpx.AsyncClient") as mock_client:
-        mock_response_obj = Mock()
-        mock_response_obj.json.return_value = mock_response
-        mock_response_obj.raise_for_status.return_value = None
-
-        mock_client.return_value.__aenter__.return_value.post = AsyncMock(
-            return_value=mock_response_obj,
-        )
-
+    with patch.object(get_api_client(), "post", return_value=mock_response):
         result = await test_func()
         assert result == mock_response
 
@@ -400,19 +525,12 @@ async def test_mock_decorator_with_expects_response(simulation_env):
 
     mock_response = {"result": {"mocked": True}}
 
-    with patch("veris_ai.tool_mock.httpx.AsyncClient") as mock_client:
-        mock_response_obj = Mock()
-        mock_response_obj.json.return_value = mock_response
-        mock_response_obj.raise_for_status.return_value = None
-
-        mock_client.return_value.__aenter__.return_value.post = AsyncMock(
-            return_value=mock_response_obj,
-        )
-
+    with patch.object(get_api_client(), "post", return_value=mock_response) as mock_request:
         result = await test_func()
         # Verify the request payload included expects_response
-        first_call = mock_client.return_value.__aenter__.return_value.post.call_args
-        assert first_call.kwargs["json"]["expects_response"] is True
+        mock_request.assert_called_once()
+        call_args = mock_request.call_args
+        assert call_args[0][1]["response_expectation"] == "required"
         assert result == {"result": {"mocked": True}}
 
 
@@ -426,19 +544,12 @@ async def test_mock_decorator_with_cache_response(simulation_env):
 
     mock_response = {"result": {"cached": True}}
 
-    with patch("veris_ai.tool_mock.httpx.AsyncClient") as mock_client:
-        mock_response_obj = Mock()
-        mock_response_obj.json.return_value = mock_response
-        mock_response_obj.raise_for_status.return_value = None
-
-        mock_client.return_value.__aenter__.return_value.post = AsyncMock(
-            return_value=mock_response_obj,
-        )
-
+    with patch.object(get_api_client(), "post", return_value=mock_response) as mock_request:
         result = await test_func(42)
         # Verify the request payload included cache_response
-        first_call = mock_client.return_value.__aenter__.return_value.post.call_args
-        assert first_call.kwargs["json"]["cache_response"] is True
+        mock_request.assert_called_once()
+        call_args = mock_request.call_args
+        assert call_args[0][1]["cache_response"] is True
         assert result == {"result": {"cached": True}}
 
 
@@ -452,20 +563,13 @@ async def test_mock_decorator_all_parameters(simulation_env):
 
     mock_response = {"status": "mocked", "values": [1, 2, 3]}
 
-    with patch("veris_ai.tool_mock.httpx.AsyncClient") as mock_client:
-        mock_response_obj = Mock()
-        mock_response_obj.json.return_value = mock_response
-        mock_response_obj.raise_for_status.return_value = None
-
-        mock_client.return_value.__aenter__.return_value.post = AsyncMock(
-            return_value=mock_response_obj,
-        )
-
+    with patch.object(get_api_client(), "post", return_value=mock_response) as mock_request:
         result = await test_func("test", 3)
         # Verify the request payload
-        first_call = mock_client.return_value.__aenter__.return_value.post.call_args
-        payload = first_call.kwargs["json"]
-        assert payload["expects_response"] is False
+        mock_request.assert_called_once()
+        call_args = mock_request.call_args
+        payload = call_args[0][1]
+        assert payload["response_expectation"] == "none"
         assert payload["cache_response"] is True
         assert result == mock_response
 
@@ -480,19 +584,12 @@ async def test_mock_decorator_function_mode_defaults_expects_response(simulation
 
     mock_response = {"result": {"mocked": True}}
 
-    with patch("veris_ai.tool_mock.httpx.AsyncClient") as mock_client:
-        mock_response_obj = Mock()
-        mock_response_obj.json.return_value = mock_response
-        mock_response_obj.raise_for_status.return_value = None
-
-        mock_client.return_value.__aenter__.return_value.post = AsyncMock(
-            return_value=mock_response_obj,
-        )
-
+    with patch.object(get_api_client(), "post", return_value=mock_response) as mock_request:
         await test_func()
         # Verify expects_response is False for function mode by default
-        first_call = mock_client.return_value.__aenter__.return_value.post.call_args
-        assert first_call.kwargs["json"]["expects_response"] is False
+        mock_request.assert_called_once()
+        call_args = mock_request.call_args
+        assert call_args[0][1]["response_expectation"] == "none"
 
 
 @pytest.mark.asyncio
@@ -504,21 +601,14 @@ async def test_mock_decorator_json_schema_in_payload(simulation_env):
     async def test_func() -> List[dict]:
         return [{"result": "real"}]
 
-    mock_response = {"result": [{"mocked": True}]}
-
-    with patch("veris_ai.tool_mock.httpx.AsyncClient") as mock_client:
-        mock_response_obj = Mock()
-        mock_response_obj.json.return_value = mock_response
-        mock_response_obj.raise_for_status.return_value = None
-
-        mock_client.return_value.__aenter__.return_value.post = AsyncMock(
-            return_value=mock_response_obj,
-        )
+    mock_response = [{"mocked": True}]
 
+    with patch.object(get_api_client(), "post", return_value=mock_response) as mock_request:
         await test_func()
         # Verify the JSON schema was included in the payload
-        first_call = mock_client.return_value.__aenter__.return_value.post.call_args
-        payload = first_call.kwargs["json"]
+        mock_request.assert_called_once()
+        call_args = mock_request.call_args
+        payload = call_args[0][1]
         return_type_schema = payload["tool_call"]["return_type"]
         # Should be a JSON string containing schema for List[dict]
         assert isinstance(return_type_schema, str)
